package ehu.student.oo.ui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.net.URL;

import javax.swing.ImageIcon;

/**
 * Un JTablero es un tipo particular de tablero que ADEMAS
 * PUEDE contener O NO un "cursor".
 * 
 * El cursor, si existe, estara situado en alguno
 * de los puntos del tablero y se representa graficamente:
 *    o bien mediante un circulo rojo,
 *    o bien mediante una palabra 
 *    o bien mediante una imagen
 */
public class JTablero extends JTableroSimple
{
	private static final long serialVersionUID = 1L;

	/*
	 * Codigos numericos que representan
	 * cada uno de los estilos disponibles
	 * para representar graficamente el cursor
	 */
	public static final int CURSOR_STYLE_CIRCLE = 0;
	public static final int CURSOR_STYLE_STRING = 1;
	public static final int CURSOR_STYLE_IMG = 2;
	public static final int CURSOR_STYLE_NONE = -1;
	
	/*
	 * Estilo de dibujo del cursor:
	 *    - si false, se dibuja un circulo
	 *    - si true, se dibuja una letra
	 */
	private int cursorStyle = CURSOR_STYLE_CIRCLE;
	
	
	/*
	 * Nombre del recurso que contiene la imagen usada para
	 * representar graficamente el cursor.
	 * 
	 * Debe ser el nombre de un archivo incluido en
	 * la carpeta "recursos" del proyecto
	 */
	private static final String CURSOR_IMG =
		"/img/Duke.gif";
		//"/img/chess/rb.gif";

	/*
	 * Palabra que se dibujara en la posicin del cursor.
	 */
	private static final String CURSOR_STRING = "java";

	/*
	 * Color usado para el dibujo del cursor
	 */
	private Color colorCursor = Color.red;


	/*
	 * Posicion del cursor (si esta situado)
	 */
	private boolean hayCursor = false;
	private int cursor_x = -1;
	private int cursor_y = -1;


	/*
	 * Factor de escala
	 * La imagen usada para representar graficamente el cursor
	 * tiene un anchura y altura proporcionales a las dimensiones
	 * de una casilla
	 */
	private static final float ESCALA_IMAGEN = 0.75F;

	/*
	 * La URL con el archivo de la imagen 
	 * a usar para representar graficamente el cursor.
	 */
	private final URL imageURL; 

	/*
	 * Imagen usada para representar graficamente el cursor.
	 */
	private Image image;

	/*
	 * Para gestionar la carga de la imagen.
	 */
	private MediaTracker tracker;


	
	/**
	 * Crea un nuevo tablero con las dimensiones establecidas por defecto.
	 */
	public JTablero()
	{    
		/*
		 * Para localizar el recurso con la imagen usada
		 * para representar graficamente el cursor
		 */
		Class<?> cl = getClass();
		imageURL = cl.getResource(CURSOR_IMG); 
		if (imageURL == null){
			String s = "No se pudo encontrar recurso: " + CURSOR_IMG;
			throw new RuntimeException(s);
		}
	}

	/**
	 * Dibujar el tablero.
	 * 
	 * @param g el contexto grafico en el cual se pinta.
	 */
	@Override
	public void paint(Graphics g)
	{
		super.paint(g);

		if (hayCursor){
			if (cursorStyle == CURSOR_STYLE_CIRCLE) 
				pintarCursorComoCirculo(g);		
			else if (cursorStyle == CURSOR_STYLE_STRING) 
				pintarCursorComoString(g, CURSOR_STRING);
			else if (cursorStyle == CURSOR_STYLE_IMG) 
				pintarCursorComoImagen(g);
		}
	}

	/**
	 * Colocar el cursor en una posicion del tablero.
	 * 
	 * @param x la abscisa de la posicion
	 * @param y la ordenada de la posicion
	 */
	public void setCursorAt(int x, int y){
		hayCursor = 0 <= x && x < getWidth() && 0 <= y && y < getHeight();
		if (hayCursor){
			cursor_x = x;
			cursor_y = y;
		}
		/*
		 * Para provocar el redibujado del tablero
		 */
		repaint();
	}

	/**
	 * Retirar el cursor del tablero.
	 * 
	 */
	public void rmvCursor(){
		hayCursor = false;
		/*
		 * Para provocar el redibujado del tablero
		 */
		repaint();
	}

	/**
	 * Devuelve true si el cursor esta situado el tablero.
	 * 
	 * @return true si el tablero contiene un cursor; 
	 * false en caso contrario
	 */
	public boolean hayCursor(){
		return hayCursor;
	}


	/**
	 * Devuelve el estilo actual para el dibujado del cursor.
	 * 
	 * @return true si el cursor se representa por una imagen;
	 * false en caso contrario
	 */
	public int isCursorStyle() {
		return cursorStyle;
	}

	/**
	 * Seleccionar el estilo actual para el dibujado del cursor.
	 * 
	 * @param s codigo indicando el estilo de dibujo del tablero.
	 */
	public void setCursorStyle(int s) {
		if (s != cursorStyle){
			cursorStyle = s;
			/*
			 * Para provocar el redibujado del tablero
			 */
			repaint();
		}
	}


	/**
	 * Dibujar el cursor mediante un circulo.
	 * 
	 * @param g el contexto grafico en el cual se pinta.
	 */
	private void pintarCursorComoCirculo(Graphics g)
	{
		int x = cursor_x;
		int y = cursor_y;
		g.setColor(colorCursor);
		g.fillOval(x-getCellWidth()/4, y-getCellHeight()/4, 
				   getCellWidth()/2, getCellHeight()/2);
	}

	/**
	 * Dibujar el cursor mediante una palabra.
	 * 
	 * @param g el contexto grafico en el cual se pinta.
	 */
	private void pintarCursorComoString(Graphics g, String s)
	{
		int x = cursor_x;
		int y = cursor_y;
		g.setColor(colorCursor);
		g.drawString(s, x, y);
	}

	/**
	 * Dibujar el cursor mediante una imagen.
	 * 
	 * @param g el contexto grafico en el cual se pinta.
	 */
	private void pintarCursorComoImagen(Graphics g)
	{
		if (image == null){
			/*
			 * La imagen solamente se carga si es ncesario.
			 */
			loadImage();
		}
		/*
		 * Calcular dimensiones de la imagen cargada.
		 */
		final int imgWidth = image.getWidth(null);
		final int imgHeight = image.getHeight(null);
		
		/*
		 * Dibujar la imagen centrada en la posicion del cursor.
		 */
		int x = cursor_x-(imgWidth/2);
		int y = cursor_y-(imgHeight/2);
		g.drawImage(image, x, y, null);
	}

	/**
	 * Cargar la imagen a usar para dibujar el cursor.
	 * 
	 * Carga la imagen del recurso correspondiente y
	 * crea una copia a escala proporcional a las dimensiones de la
	 * casilla.
	 */
	private void loadImage() {
		/*
		 * Para cargar la imagen
		 */
		ImageIcon imageIcon = new ImageIcon(imageURL);
		image = imageIcon.getImage();

		/*
		 * Para crear una copia a escala
		 * proporcional a las dimensiones de la casilla
		 */
		final int w = (int)(ESCALA_IMAGEN * getCellWidth());
		final int h = (int)(ESCALA_IMAGEN * getCellHeight());
		image = image.getScaledInstance(w, h, -4);

		/*
		 * Para asegurar que la imagen 
		 * ha sido cargada por completo antes de seguir.
		 */
		waitForImage(image, CURSOR_IMG + "{scaled}");
	}

	/**
	 * Asegurar que una imagen dada se ha cargado por completo.
	 * 
	 * @param imagen la imagen referida 
	 * @param msg texto informativo a usar si se produce un error.
	 */
	private void waitForImage(Image imagen, String msg) {
		if (tracker == null) tracker = new MediaTracker(this);

		int id = 0;
		tracker.addImage(imagen, id);
		try {
			tracker.waitForID(id);
		} catch (InterruptedException e) {
			throw new RuntimeException(e);
		}
		if (tracker.isErrorAny()){
			String s = "Error cargando imagen{" + msg + "}";
			throw new RuntimeException(s);

		}
	}
}